library(shiny)
library(QCA)
library(QCAGUI)
# options(shiny.maxRequestSize=30*1024^2) # 30MB per file
# options(warn=-1)

setwd(Sys.getenv("userwd"))

## Future feature: local settings

# if (file.exists(noedit)) {
#     settings <- strsplit(readLines(noedit), split=" = ")
#     opts <- unlist(lapply(settings, "[[", 1))
#     vals <- unlist(lapply(settings, "[[", 2))
# }




listFiles <- function(dirpath, filetype="*") {
    
    result <- list(dirs = NULL, files = NULL, filepath = as.matrix(filepath), ok = oktoset)
    
    # get all files
    found <- list.files(dirpath)
    
    temp <- toupper(found)
    found <- found[match(sort(temp), temp)]
    
    if (length(found) > 0) {
        
        isdir <- sapply(found, function(x) file_test("-d", file.path(dirpath, x)))
        
        if (any(isdir)) {
            result$dirs <- found[isdir]
        }
        
        if (any(!isdir)) {
            if (filetype != "*") {
                extensions <- unlist(lapply(strsplit(found, split="\\."), "[", 2))
                found <- found[which(toupper(extensions) == toupper(filetype))]
                if (length(found) > 0) {
                    result$files <- found
                }
            }
            else {
                result$files <- found[!isdir]
            }
        }
        
        # this is necessary because Javascript interprets an array of length 1
        # as a singleton (not an array anymore)
        
        if (length(result$dirs) == 1) {
            result$dirs <- as.matrix(result$dirs)
        }
        
        if (length(result$files) == 1) {
            result$files <- as.matrix(result$files)
        }
        
    }
    
    # result$filename <- as.matrix(unlist(strsplit(basename(filepath), split="\\."))[1])
    if (!identical(filepath, "")) {
        if (file_test("-f", filepath)) {
            
            resfilename <- unlist(strsplit(basename(filepath), split="\\."))
            extension <<- resfilename[length(resfilename)]
            resfilename <- resfilename[-length(resfilename)]
            resfilename <- gsub(" ", "", resfilename)
            
            if (length(resfilename) > 1) {
                resfilename <- paste(filename[-length(resfilename)], collapse=".")
            }
            
            
            if (length(resfilename) > 0) {
                foo <- unlist(strsplit(resfilename, split=""))
                if (!is.na(suppressWarnings(as.numeric(foo[1])))) {
                    resfilename <- paste("x", resfilename, sep="")
                }
            }
            
            filename <<- resfilename
            
        }
    }
    
    
    result$filename <- filename
    result$extension <- extension
    result$wd <- getwd()
    
    return(result)
    
}



mydata <- NULL



current_path <- getwd()
oktoset <- TRUE
filepath <- ""
filename <- ""
extension <- ""
vert <- NULL



# Define server logic for random distribution application
shinyServer(function(input, output, session) {
    
    
    
    # only once at a time, when the "Load" button is pressed
    observe({
        input$choose
        session$sendCustomMessage(type = "dirfile", listFiles(current_path))
    })
    
    
    observe({
        input$dirfilist
        session$sendCustomMessage(type = "dirfile", listFiles(current_path))
    })
    
    
    # import csv files
    observe({
        
        # this changes every time the Javascript object "read_table" changes
        read_table <- input$read_table
        
        filepath <<- ""
        
        oktoset <- TRUE
            
        
        if (!is.null(input$dirfile_chosen)) {
            
            dfchosen <- input$dirfile_chosen
            oktoset <<- TRUE
            
            # run this each time the "dirfile_chosen" vector changes, see Javascript code
            
            if (dfchosen[1] == "file") {
                filepath <<- file.path(current_path, dfchosen[2])
            }
            else {
                splitpath <- unlist(strsplit(current_path, split=.Platform$file.sep))
                
                if (dfchosen[2] == ".." | dfchosen[2] == "...") {
                    
                    if (length(splitpath) > 1) {
                        splitpath <- splitpath[-length(splitpath)]
                    }
                    
                    if (identical(splitpath, "")) {
                        splitpath <- "/"
                    }
                    
                    pathtobe <- paste(splitpath, collapse=.Platform$file.sep)
                    
                    if (file_test("-x", pathtobe)) {
                        current_path <<- pathtobe
                    }
                    else {
                        oktoset <<- FALSE
                    }
                }
                else {
                    current_dirs <- listFiles(current_path)$dirs
                    
                    if (dfchosen[2] %in% current_dirs) {
                        pathtobe <- file.path(current_path, dfchosen[2])
                        if (file_test("-x", pathtobe)) {
                            current_path <<- pathtobe
                        }
                        else {
                            oktoset <<- FALSE
                        }
                    }
                    else {
                        
                        if (dfchosen[2] == "root") {
                            dfchosen[2] <- ""
                        }
                        
                        splitpath <- splitpath[seq(which(splitpath == dfchosen[2]))]
                        pathtobe <- ifelse(length(splitpath) == 1,
                                           ifelse(identical(splitpath, ""), "/", splitpath),
                                           paste(splitpath, collapse=.Platform$file.sep))
                        
                        if (file_test("-x", pathtobe)) {
                            current_path <<- pathtobe
                        }
                        else {
                            oktoset <<- FALSE
                        }
                    }
                }
            }
            
            if (oktoset) {
                if (!grepl("/", current_path)) {
                    current_path <- paste(current_path, "/", sep="")
                }
            }
            
            setwd(current_path)
            session$sendCustomMessage(type = "dirfile", listFiles(current_path))
                
        }
        else {
            current_path <<- getwd()
        }
        
        
        if (!identical(filepath, "")) {
            
            numevars <- ""
            
            header <- read_table$header
            colsep <- read_table$sep # comma separated by default
            row_names <- read_table$row_names
            decimal <- read_table$dec
            
            filename <<- unlist(strsplit(basename(filepath), split="\\."))
            filename <<- filename[-length(filename)]
            if (length(filename) > 1) {
                filename <<- paste(filename, collapse=".")
            }
            
            
            if (row_names != "") { # this isn't a vector, just a name or a number
                if (!is.na(suppressWarnings(as.numeric(row_names)))) {
                    row_names <- as.numeric(row_names)
                }
                tc <- capture.output(tryCatch(read.table(filepath, header=header, ifelse(colsep == "tab", "\t", colsep),
                          row.names=row_names, as.is=TRUE, dec=decimal, nrows = 2), error = function(e) e, warning = function(w) w))
                   
            }
            else {
                tc <- capture.output(tryCatch(read.table(filepath, header=header, ifelse(colsep == "tab", "\t", colsep),
                          as.is=TRUE, dec=decimal, nrows = 2), error = function(e) e, warning = function(w) w))
            }
            
            if (any(grepl("subscript out of bounds", tc))) {
                mesaj <- paste("The data doesn't have ", row_names, " columns.", sep = "")
                session$sendCustomMessage(type = "datainfo", list(ncols=1, nrows=1, colnames=mesaj, rownames="error!"))
                return(invisible())
            }
            else if (any(grepl("are not allowed", tc))) {
                mesaj <- paste("The row.names column has duplicated values.", sep = "")
                session$sendCustomMessage(type = "datainfo", list(ncols=1, nrows=1, colnames=mesaj, rownames="error!"))
                return(invisible())
            }
            else if (any(grepl("data frame with 0 columns", tc))) {
                mesaj <- paste("The data has only 1 column.", sep = "")
                tc <- tryCatch(read.table(filepath, header=header, ifelse(colsep == "tab", "\t", colsep),
                           as.is=TRUE, dec=decimal, nrows = 2), error = function(e) e)
                session$sendCustomMessage(type = "datainfo", list(ncols=2, nrows=2, colnames=c(colnames(tc), mesaj), rownames=""))
                return(invisible())
            }
            else if (any(grepl("attempt to select less than one element", tc))) {
                mesaj <- paste("The column \"", row_names, "\" was not found.", sep = "")
                session$sendCustomMessage(type = "datainfo", list(ncols=1, nrows=1, colnames=mesaj, rownames="error!"))
                return(invisible())
            }
            
            
            tc <- tryCatch(read.table(filepath, header=header, ifelse(colsep == "tab", "\t", colsep),
                           as.is=TRUE, dec=decimal, nrows = 2), error = function(e) e, warning = function(w) w)
            
            
            tcisdata <- TRUE
            
            
            if (is.null(dim(tc))) {
                if (is.list(tc)) {
                    if (identical(names(tc), c("message", "call"))) {
                        tcisdata <- FALSE
                        session$sendCustomMessage(type = "datainfo", list(ncols=1, nrows=1, colnames=tc$message, rownames=""))
                    }
                }
            }
            else {
                if (grepl("X.PDF", names(tc)[1])) {
                    tcisdata <- FALSE
                    session$sendCustomMessage(type = "datainfo", list(ncols=1, nrows=1, colnames="not a dataframe, this is a PDF file", rownames=""))
                }
            }
            
            if (tcisdata) {
                    tc <- tryCatch(read.table(filepath, header=header, ifelse(colsep == "tab", "\t", colsep),
                              as.is=TRUE, dec=decimal, nrows = 2), error = function(e) e, warning = function(w) w)
            
                if (row_names != "") {
                    tc <- tryCatch(read.table(filepath, header=header, ifelse(colsep == "tab", "\t", colsep),
                              row.names=row_names, as.is=TRUE, dec=decimal), error = function(e) e, warning = function(w) w)
                }
                else {
                    tc <- tryCatch(read.table(filepath, header=header, ifelse(colsep == "tab", "\t", colsep),
                              as.is=TRUE, dec=decimal), error = function(e) e, warning = function(w) w)
                }
                
                
                if (identical(names(tc), c("message", "call"))) {
                    # not a mistake, it might happen that a warning is issued even at this stage
                    session$sendCustomMessage(type = "datainfo", list(ncols=1, nrows=1, colnames=tc$message, rownames=""))
                }
                else {
                
                    mydata <<- tc
                    set.seed(12345)
                    vert <<- sample(185:200, nrow(mydata), replace = TRUE)
                    
                    numerics <- as.vector(unlist(lapply(mydata, possibleNumeric)))
                    calibrated <- as.vector(unlist(lapply(mydata, function(x) {
                        all(na.omit(x) >= 0 & na.omit(x) <= 1)
                    })))
                    
                    session$sendCustomMessage(type = "datainfo", list(ncols=ncol(mydata),
                                                                      nrows=nrow(mydata),
                                                                      colnames=colnames(mydata),
                                                                      rownames=rownames(mydata),
                                                                      numerics=numerics,
                                                                      calibrated=calibrated))
                    rowend <- min(19, nrow(mydata))
                    colend <- min(10, ncol(mydata))
                    
                    
                    tosend <- as.list(mydata[seq(rowend), seq(colend)])
                    names(tosend) <- NULL # to make it look like a simple Array in Javascript
                    
                    session$sendCustomMessage(type = "theData", list(tosend, paste(1, 1, rowend, colend, sep="")))
                }
            }
        }
        
    })
    
    
    
    observe({
        scrollvh <- input$scrollvh
        
        if (!is.null(scrollvh)) {
            
            rowstart <- scrollvh[1] + 1
            colstart <- scrollvh[2] + 1
            rowend <- min(scrollvh[1] + scrollvh[3], nrow(mydata))
            colend <- min(scrollvh[2] + scrollvh[4], ncol(mydata))
            
            tosend <- as.list(mydata[seq(rowstart, rowend), seq(colstart, colend)])
            names(tosend) <- NULL
            
            # session$sendCustomMessage(type = "theData", tosend)
            session$sendCustomMessage(type = "theData", list(tosend, paste(rowstart, colstart, rowend, colend, sep="")))
        }   
    })
    
    
    
    
    # eqmcc
    observe({
        
        foo <- input$eqmcc2R
        
        if (!is.null(mydata)) {
            
            outc <- c("")
            if (length(foo$outcome) > 0) {
                outc <- unlist(foo$outcome)
            }
            
            cnds <- c("")
            if (length(foo$conditions) > 0) {
                cnds <- unlist(foo$conditions)
            }
            
            expl <- c("1")
            if (length(foo$explain) > 0) {
                expl <- unlist(foo$explain)
            }
            
            incl <- c("")
            if (length(foo$include) > 0) {
                incl <- unlist(foo$include)
            }
            
            direxp <- c()
            if (length(foo$dir_exp) > 0) {
                direxp <- unlist(foo$dir_exp)
                if (all(direxp == "-")) {
                    direxp <- c()
                }
            }
            
            textoutput <- capture.output(tryCatch(
                eqmcc(mydata, outcome = outc,
                      neg.out = foo$neg_out,
                      conditions = cnds,
                      relation = foo$relation,
                      n.cut = as.numeric(foo$n_cut),
                      incl.cut1 = as.numeric(foo$incl_cut1),
                      incl.cut0 = as.numeric(foo$incl_cut0),
                      explain = expl,
                      include = incl,
                      min.dis = foo$min_dis,
                      dir.exp = direxp,
                      details = foo$details,
                      show.cases = foo$show_cases,
                      use.tilde = foo$use_tilde,
                      use.letters = foo$use_letters,
                      via.web=TRUE) , error = function(e) e)
            )
            
            if (any(error <- grepl("Error", textoutput))) {
                errmessage <- paste0("Error:", unlist(strsplit(textoutput[which(error)], split=":"))[2])
                errmessage <- substr(errmessage, 1, nchar(errmessage) - 1)
                textoutput <- c("error", errmessage, "")
            }
            
            textoutput <- gsub("undefined columns selected>", "Column names in the command don't match those in the interface.", textoutput)
            
            session$sendCustomMessage(type = "eqmcc", list(textoutput))
        }
        
    })
    
    
    
    
    
    # truth table
    observe({
        
        foo <- input$tt2R
        
        if (!is.null(mydata)) {
            
            outc <- c("")
            if (length(foo$outcome) > 0) {
                outc <- unlist(foo$outcome)
            }
            
            cnds <- c("")
            if (length(foo$conditions) > 0) {
                cnds <- unlist(foo$conditions)
            }
            
            sortbys <- c("")
            if (length(foo$sort_by) > 0) {
                sortbys <- unlist(foo$sort_by)
            }
            
            decrbys <- c()
            if (length(foo$decr_by) > 0) {
                decrbys <- unlist(foo$decr_by)
            }
            
            textoutput <- capture.output(tryCatch(
                truthTable(mydata, outcome = outc,
                      neg.out = foo$neg_out,
                      conditions = cnds,
                      n.cut = as.numeric(foo$n_cut),
                      incl.cut1 = as.numeric(foo$incl_cut1),
                      incl.cut0 = as.numeric(foo$incl_cut0),
                      complete = foo$complete,
                      show.cases = foo$show_cases,
                      sort.by = sortbys,
                      decreasing = decrbys,
                      use.letters = foo$use_letters), error = function(e) e)
            )
            
            
            
            if (any(error <- grepl("Error", textoutput))) {
                errmessage <- paste0("Error:", unlist(strsplit(textoutput[which(error)], split=":"))[2])
                errmessage <- substr(errmessage, 1, nchar(errmessage) - 1)
                textoutput <- c("error", errmessage, "")
            }
            
            textoutput <- gsub("undefined columns selected>", "Column names in the command don't match those in the interface.", textoutput)
            session$sendCustomMessage(type = "tt", list(textoutput))
        }
        
    })
    
    
    
    observe({
        thinfo <- input$thinfo
        
        if (!is.null(thinfo)) {
            if (thinfo[2] != "") {
                
                # make sure the variable is numeric
                if (any(is.na(suppressWarnings(as.numeric(mydata[, thinfo[2]]))))) {
                    response <- "notnumeric"
                }
                else {
                    response <- findTh(mydata[, thinfo[2]], groups = as.numeric(thinfo[1]) + 1)
                }
                
                session$sendCustomMessage(type = "thvalsfromR", 
                    as.list(response)
                )
            }
        }
    })
    
    
    # export to file
    observe({
        exportobj <- input$exportobj
        
        if (!is.null(exportobj)) {
            
            filesep <- exportobj$sep
            if (filesep == "tab") {
                filesep <- "\t"
            }
            
            separator <- exportobj$sep
            filetowrite <- file.path(current_path, exportobj$filename)
            
            if (exportobj$newfile) {
                if (exportobj$filename != "") {
                    filetowrite <- file.path(current_path, exportobj$filename)
                }
            }
            
            export(mydata, filetowrite, sep=filesep, col.names=exportobj$header, caseid=exportobj$caseid)
        }
    })
    
    
    
    # calibrate
    observe({
        foo <- input$calibrate
        
        if (!is.null(foo)) {
            
            checks <- rep(TRUE, 9)
            
            checks[1] <- !is.null(mydata)
            # checks[2] <- length(foo$thresholds) > 0
            
            foo$thresholds <- unlist(foo$thresholds)
            
            
            nms <- unlist(foo$thnames)[foo$thresholds != ""]
            foo$thresholds <- foo$thresholds[foo$thresholds != ""]
            
            
            thrs <- suppressWarnings(as.numeric(foo$thresholds))
            if (any(!is.na(thrs))) {
                foo$thresholds <- as.numeric(thrs[!is.na(thrs)])
            }
            
            # if (checks[2]) {
            #     checks[3] <- all(!is.na(suppressWarnings(as.numeric(foo$thresholds))))
            # }
            
            # if (checks[3]) {
            #     foo$thresholds <- as.numeric(foo$thresholds)
            # }
            
            if (all(foo$thresholds == "")) {
                foo$thresholds <- NA
            }
            else {
                if (!is.null(nms)) {
                    if (length(nms) == length(foo$thresholds)) {
                        names(foo$thresholds) <- nms
                    }
                    else {
                        foo$thresholds <- NA
                    }
                }
            }
            
            foo$x <- unlist(foo$x)
            
            checks[4] <- !is.null(foo$x)
            if (checks[4]) {
                checks[5] <- foo$x != ""
            }
            
            
            if (checks[1] & checks[5]) {
                
                if (foo$x %in% names(mydata)) {
                    checks[6] <- is.numeric(mydata[, foo$x])
                }
            }
            
            checks[7] <- !is.na(suppressWarnings(as.numeric(foo$idm)))
            if (checks[7]) {
                foo$idm <- as.numeric(foo$idm)
            }
            
            checks[8] <- !is.na(suppressWarnings(as.numeric(foo$p)))
            if (checks[8]) {
                foo$p <- as.numeric(foo$p)
            }
            
            checks[9] <- !is.na(suppressWarnings(as.numeric(foo$q)))
            if (checks[9]) {
                foo$q <- as.numeric(foo$q)
            }
            
            scrollvh <- unlist(foo$scrollvh)
            scrollvh[1:2] <- scrollvh[1:2] + 1
            
            if (all(checks)) {
                
                textoutput <- capture.output(tryCatch(
                    calibrate(
                        mydata[, foo$x],
                        type = foo$type,
                        thresholds = foo$thresholds,
                        include = foo$include,
                        logistic = foo$logistic,
                        idm = foo$idm,
                        ecdf = foo$ecdf,
                        p = foo$p,
                        q = foo$q), error = function(e) e)
                )
                
                response <- vector(mode="list", length = 3)
                
                if (any(error <- grepl("Error", textoutput))) {
                    errmessage <- paste0("Error:", unlist(strsplit(textoutput[which(error)], split=":"))[2])
                    errmessage <- substr(errmessage, 1, nchar(errmessage) - 1)
                    textoutput <- c("error", errmessage, "")
                }
                else {
                    textoutput <- "no problem"
                    
                    mydata[, ifelse(foo$newvar != "", foo$newvar, foo$x)] <<- calibrate(
                            mydata[, foo$x],
                            type = foo$type,
                            thresholds = foo$thresholds,
                            include = foo$include,
                            logistic = foo$logistic,
                            idm = foo$idm,
                            ecdf = foo$ecdf,
                            p = foo$p,
                            q = foo$q)
                    
                    rowstart <- scrollvh[1]
                    colstart <- scrollvh[2]
                    rowend <- min(scrollvh[1] + scrollvh[3], nrow(mydata))
                    if (nrow(mydata) > rowend) {
                        rowend <- rowend + 1
                    }
                    colend <- min(scrollvh[2] + scrollvh[4], ncol(mydata))
                    if (ncol(mydata) > colend) {
                        colend <- colend + 1
                    }
                    
                    tosend <- as.list(mydata[seq(rowstart, rowend), seq(colstart, colend)])
                    names(tosend) <- NULL
                    
                    numerics <- as.vector(unlist(lapply(mydata, possibleNumeric)))
                    response[[2]] <- list(ncols=ncol(mydata),
                                          nrows=nrow(mydata),
                                          colnames=colnames(mydata),
                                          rownames=rownames(mydata),
                                          numerics=numerics)
                    response[[3]] <- list(tosend, paste(rowstart, colstart, rowend, colend, sep=""))
                    
                    
                }
                
                response[[1]] <- as.list(textoutput)
                session$sendCustomMessage(type = "calibrate", response)
                
            }
        }
        
    })
    
    
    
    # dataModif
    observe({
        val <- input$dataModif
        if (!is.null(val)) {
            if (val[1] == "r") {
                rownames(mydata)[as.numeric(val[2])] <<- val[3]
            }
            else if (val[1] == "c") {
                colnames(mydata)[as.numeric(val[2])] <<- val[3]
            }
            else {
                valn <- suppressWarnings(as.numeric(val[3]))
                mydata[as.numeric(val[1]), as.numeric(val[2])] <<- ifelse(!is.na(valn), valn, val[3])
            }
        }
    })
    
    
    
    # dataPoints
    observe({
        foo <- input$thsetter2R
        if (!is.null(foo)) {
            
            cond <- unlist(foo$cond)
            
            if (cond %in% names(mydata)) {
                horz <- sort(mydata[, cond])
                response <- cbind(horz[!is.na(horz)], vert[!is.na(horz)])
                session$sendCustomMessage(type = "dataPoints", response)
            }
        }
    })
    
    
    
    # XYplotPoints
    observe({
        foo <- input$xyplot
        
        if (!is.null(foo)) {
            if (all(c(foo$x, foo$y) %in% names(mydata))) {
                
                x <- mydata[, foo$x]
                y <- mydata[, foo$y]
                
                
                rpof <- list(pof(x, y, ron = TRUE),
                             pof(1 - x, y, ron = TRUE),
                             pof(x, 1 - y, ron = TRUE),
                             pof(1 - x, 1 - y, ron = TRUE))
                
                rpof <- lapply(rpof, function(x) {
                    
                    frmted <- formatC(c(x$incl.cov$incl, x$incl.cov$cov.r), format="f", digits = 3)
                    if (!is.null(x$optionals[, "ron"])) {
                        frmted <- c(frmted, formatC(x$optionals[, "ron"], format="f", digits = 3))
                    }
                    return(frmted)
                })
                
                response = list(rownames(mydata),
                                mydata[, foo$x],
                                mydata[, foo$y],
                                rpof)
                session$sendCustomMessage(type = "xyplot", response)
                
            }
        }
    })
    
    
    
    
    # recode
    observe({
        foo <- input$recode
        
        if (!is.null(foo)) {
            
            checks <- rep(TRUE, 3)
            
            checks[1] <- !is.null(mydata)
            
            foo$x <- unlist(foo$x)
            
            checks[2] <- !is.null(foo$x)
            
            if (checks[2]) {
                checks[3] <- foo$x != ""
            }
            
            
            foo$oldv <- unlist(foo$oldv)
            foo$newv <- unlist(foo$newv)
            uniques <- unique(foo$newv)
            
            rules <- ""
            for (i in seq(length(uniques))) {
                part <- paste(paste(foo$oldv[foo$newv == uniques[i]], collapse = ","), uniques[i], sep="=")
                rules <- paste(rules, part, ifelse(i == length(uniques), "", "; "), sep="")
            }
            
            
            scrollvh <- unlist(foo$scrollvh)
            scrollvh[1:2] <- scrollvh[1:2] + 1
            
            if (all(checks)) {
                
                textoutput <- capture.output(tryCatch(
                    recode(mydata[, foo$x],
                           rules = rules), error = function(e) e)
                )
                
                response <- vector(mode="list", length = 3)
                
                if (any(error <- grepl("Error", textoutput))) {
                    errmessage <- paste0("Error:", unlist(strsplit(textoutput[which(error)], split=":"))[2])
                    errmessage <- substr(errmessage, 1, nchar(errmessage) - 1)
                    textoutput <- c("error", errmessage, "")
                }
                else {
                    textoutput <- "no problem"
                    
                    mydata[, ifelse(foo$newvar != "", foo$newvar, foo$x)] <<- recode(
                            mydata[, foo$x],
                            rules = rules)
                    
                    rowstart <- scrollvh[1]
                    colstart <- scrollvh[2]
                    rowend <- min(scrollvh[1] + scrollvh[3], nrow(mydata))
                    if (nrow(mydata) > rowend) {
                        rowend <- rowend + 1
                    }
                    colend <- min(scrollvh[2] + scrollvh[4], ncol(mydata))
                    if (ncol(mydata) > colend) {
                        colend <- colend + 1
                    }
                    
                    tosend <- as.list(mydata[seq(rowstart, rowend), seq(colstart, colend)])
                    names(tosend) <- NULL
                    
                    numerics <- as.vector(unlist(lapply(mydata, possibleNumeric)))
                    response[[2]] <- list(ncols=ncol(mydata),
                                          nrows=nrow(mydata),
                                          colnames=colnames(mydata),
                                          rownames=rownames(mydata),
                                          numerics=numerics)
                    response[[3]] <- list(tosend, paste(rowstart, colstart, rowend, colend, sep=""))
                    
                    
                }
                
                response[[1]] <- as.list(textoutput)
                session$sendCustomMessage(type = "recode", response)
                
            }
        }
        
    })
  
})